/*
 * Config-file parser.
 *
 * (c)1999-2000 Frank van Viegen
 * This code is subject to the GPL. Read COPYING for details.
 */


%{
    #include <stdio.h>
    #include <string.h>
    
    #include "irmanager.h"
    #include "configparse.h"
    #include "configread.h"

    extern int yylineno;
    
    #define YYDEBUG 1
    #define YYERROR_VERBOSE
    
%}

%union {
    double valdbl;
    long valint;
    char *valstr;
    struct mapstr *valmap;
    struct keystr *valkey;
}

%token <valint> NUMINT
%token <valstr>	STRING
%token <valstr> SYMBOL
%token <valstr> CODE

%token DEFAULTMAP
%token MAP
%token KEY
%token DIRECTKEY
%token COMMAND
%token SENDFIFO
%token EXPIRE
%token TERMINATE
%token TRUE
%token FALSE
%token ACTIVATE
%token FIFO
%token SIGNAL
%token DIRECT
%token EXIT
%token SERIAL
%token PTY
%token NAMEDPIPE
%token WRITEFIFO
%token INIT

%type <valmap> mapblock
%type <valkey> key
%type <valkey> keyblock

%start config

%%

config:		/* empty */		{}
		| statement config	{}
;

statement:	MAP SYMBOL '{' mapblock '}'	{ $4->name = $2; }
		| key				{ addKey(globalMap,$1); }
		| DEFAULTMAP '=' SYMBOL ';'	{ setDefaultMapName($3); }
		| SIGNAL SYMBOL '=' CODE ';'	{ newSignal($2,(ircode *)$4); }
		| FIFO SYMBOL '=' STRING ';'	{ newFifo($2,$4,0); }
		| NAMEDPIPE SYMBOL '=' STRING ';' { newFifo($2,$4,1); }
		| PTY SYMBOL '=' STRING ';'	{ newFifo($2,$4,2); }
		| SERIAL '=' STRING ';'		{ setSerial($3); }
		| INIT '{' keyblock '}' 	{ setInit($3->action); }
		| ';'				{}
;

mapblock:	/* empty */			{ $$ = newMapstr(); }
		| key mapblock			{ addKey($2,$1); $$=$2; }
		| EXPIRE '=' NUMINT ';' mapblock	{ $5->expire=$3; $$=$5; }
		| TERMINATE '=' TRUE ';' mapblock	{ $5->terminate=-1; $$=$5; }
		| TERMINATE '=' FALSE ';' mapblock	{ $5->terminate=0; $$=$5; }
		| ';' mapblock			{ $$ = $2; }
;

key:		KEY SYMBOL '{' keyblock '}'	{ $4->codename=$2; $$=$4; }
		| KEY '*' '{' keyblock '}'	{ $$=$4; }
;

keyblock:	/* empty */			{ $$ = newKeystr(); }
		| COMMAND STRING ';' keyblock	{ addKeyaction($4,CMD_COMMAND,$2,NULL); $$=$4; }
		| ACTIVATE SYMBOL ';' keyblock	{ addKeyaction($4,CMD_ACTIVATE,$2,NULL); $$=$4; }
		| SENDFIFO SYMBOL ';' keyblock	{ addKeyaction($4,CMD_SENDFIFO,$2,NULL); $$=$4; }
		| SENDFIFO SYMBOL SYMBOL ';' keyblock { addKeyaction($5,CMD_SENDFIFO,$2,$3); $$=$5; }
		| WRITEFIFO SYMBOL STRING ';' keyblock { addKeyaction($5,CMD_WRITEFIFO, $2, $3); $$=$5; }
		| EXIT ';' keyblock		{ addKeyaction($3,CMD_EXIT,NULL,NULL); $$=$3; }
		| DIRECT '=' TRUE ';' keyblock	{ $5->direct=-1; $$=$5; }
		| DIRECT '=' FALSE ';' keyblock	{ $5->direct=0; $$=$5; }
		| ';' keyblock			{ $$ = $2; }
;


%%


int yyerror(char *msg)
{
    fatalerror("%s at %s",msg,pos2text());
}
