/*
 * This service-routines for the Bison parser create the in-memory
 * configuration structures.
 *
 * (c)1999-2000 Frank van Viegen
 * This code is subject to the GPL. Read COPYING for details.
 */


#include "irmanager.h"
#include "configparse.h"
#include "options.h"
#include "stringtrans.h"


struct mapstr *firstMap = NULL;
struct fifostr *firstFifo = NULL;
struct codestr *firstCode = NULL;
char *defaultMapName = NULL;
struct mapstr *defaultMap = NULL;
struct mapstr *globalMap;
struct keyactionstr *initActionblock = NULL;

void setInit(struct keyactionstr *k)
{
	struct keyactionstr *t = initActionblock;
	if (initActionblock) {
		while(t->next) t = t->next;
		t->next = k;
	}
	else {
		initActionblock = k;
	}
}

void addKey(struct mapstr *m, struct keystr *k)
{
	k->next = m->key;
	m->key = k;
}

void setDefaultMapName(char *n)
{
	defaultMapName = n;
}

struct mapstr *newMapstr()
{
	struct mapstr *m;
	m = (struct mapstr *)malloc(sizeof (struct mapstr));
	m->name = NULL;
	m->expire = 0;
	m->terminate = 0;
	m->key = NULL;
	m->next = firstMap;
	firstMap = m;
	return(m);
}

struct keystr *newKeystr()
{
	struct keystr *k;
	k = (struct keystr *)malloc(sizeof (struct keystr));
	k->codename = NULL;
	k->direct = 0;
	k->action = NULL;	
}

void addKeyaction(struct keystr *k, int cmdtype, char *op1, char *op2)
{
	struct keyactionstr *a;
	char *c;
	a = (struct keyactionstr *)malloc(sizeof (struct keyactionstr));
	if (cmdtype==CMD_WRITEFIFO) {
		c = transString(op2);
		if (c) {
			free(op2);
			op2 = c;
		}
		else warning("unable to translate writefifo string at %s",pos2text());
	}
	a->cmdtype = cmdtype;
	a->op1 = op1;
	a->op2 = op2;
	a->next = k->action;
	k->action = a;
}

void newFifo(char *name, char *filename, int type)
{
	struct fifostr *f;
	f = (struct fifostr *)malloc(sizeof (struct fifostr));
	f->name = name;
	f->filename = filename;
	f->next = firstFifo;
	f->type = type;
	firstFifo = f;
}

void newSignal(char *name, ircode *code)
{
	struct codestr *c;
	c = (struct codestr*)malloc(sizeof (struct codestr));
	c->name = name;
	codecpy(c->code,code);
	free(code);
	c->next = firstCode;
	firstCode = c;
}

void initConfigParse()
{
	globalMap = newMapstr();
	globalMap->name = "global";
	initStringTrans(optterm);
}


void setSerial(char *s)
{
	if (!optserial) optserial = s;
}

void finishConfigParse()
{
	struct mapstr *m;
	if (!optserial) {
		optserial = "/dev/ttyS0";
		warning("serial port for IRMan device not specified - defaulting to %s",optserial);
	}

	if (defaultMapName!=NULL) {
		m = firstMap;
		while(m) {
			if (!strcasecmp(m->name,defaultMapName)) defaultMap = m;
			m = m->next;
		}
	}
	
	if (defaultMap==NULL) {
		defaultMap = newMapstr();
		warning("default map not specified - empty map will be used");
	}
	finishStringTrans();
}
