/*
 * This ties all the ends together.
 *
 * (c)1999-2000 Frank van Viegen
 * This code is subject to the GPL. Read COPYING for details.
 */


#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <fcntl.h>
#include <signal.h>
#include <sched.h>
#include <string.h>
#include <errno.h>

#include "irmanager.h"
#include "options.h"
#include "configparse.h"
#include "codehandle.h"
#include "text.h"
#include "configread.h"

static int exitFromInt(int i);
static int suspendFromInt(int i);
char logtimedat[40];

FILE *outfd = NULL;
FILE *logfd = NULL;
FILE *dumpfd = NULL;

int main(int argc, char *argv[])
{
	extern FILE *yyin;
	extern int yydebug;
	ircode code;
	int c;
	
	#ifdef DEBUGBISON
		yydebug=-1;
	#else
		yydebug=0;
	#endif
	
	outfd = stderr;
	
	/* Parse commandline options */
	parseCmdLine(argc,argv);
	
	/* Handle program termination */
	regSignalSelect(SIGINT, exitFromInt);
	regSignalSelect(SIGQUIT, exitFromInt);
	regSignalSelect(SIGTERM, exitFromInt);
	regSignalSelect(SIGTSTP, suspendFromInt);

	/* Close all open files, except stderr and stdout */
	for(c=3; c<NOFILE; c++) close(c);
	/* Close stdin if we're not in test mode */
	if (!opttestmode) close(0);
	
	/* Let's open the logfile, if one was specified. */
	if (optlogfile) {
		if (!strcmp(optlogfile,"-")) logfd = stdout;
		else {
			logfd = fopen(optlogfile, "a");
			if (!logfd) warning("unable to open/create logfile '%s'",optlogfile);
		}
		loginfo("%s\n",versiontext);
		loginfo("  Initializing\n");
	}

	/* Parse the configuration file */
	initConfigParse();
	if (optconfig!=NULL-1) {
		if (!(yyin = fopen(optconfig,"r"))) fatalerror("unable to open config-file '%s'",optconfig);
		setConfigName(optconfig);
		yyparse();
		fclose(yyin);
	}
	finishConfigParse();

	/* Become a daemon. At this point, we must do the forking and the like, since
	 * we don't want to change PID after initIrman has created a lock-file. Closing
	 * open files and changing the current directory will happen after we've inited.
         */
	if (!optnodaemon) {
		if (fork()) {		/* Make sure we're NOT a session leader */
                	wait(NULL);
			exit(0);
	    	}
	    	setsid();		/* Create a new session; a controlling terminal is assigned */
	    	if (fork()) {		/* Get rid of the controlling terminal */
			wait(NULL);
			exit(0);
	    	}
	}

	/* This'll prepare everything. We don't have to worry about any 'events' coming
	 * in while the system is not yet set up properly, because events will be
	 * generated by the mainloop.
	 */
	initIoSelect();
	if (opttestmode) 
	    initKeyboard(handleCode);
	else
	    initIrman(optserial, handleCode);
	initCodeHandle();

	/* We've now come to the point that all initializing has been done, so we can cut
	 * all connections to the outside world to become a nice quiet daemon.
	 */
	chdir("/");
	if (logfd!=stdout && dumpfd!=stdout) close(1);
	outfd = NULL;
	close(2);

	/* Allow our parent and grand-parent to die */
	if (!optnodaemon)
		kill(getppid(),3);
		
	/* Execute any initial actionblocks */
	if (initActionblock) {
		loginfo("    Initial actionblock\n");
		if (execActionList(initActionblock,NULL)) fatalerror("'exit' command in 'init' makes no sense");
	}
	
	loginfo("    Waiting for events\n");

	/* Now we can just wait for things to happen! */
	c=ioSelect();
	if (c) fatalerror("ioselect error (%s)",strerror(c));

	/* Since I'm cleaning up code anyway, why not arrange for a clean exit?! */
	finishCodeHandle();
	if (opttestmode)
	    finishKeyboard();
	else
	    finishIrman();
	finishIoSelect();
	loginfo("Terminating\n");
	if (logfd) fclose(logfd);

	return(0);
}


static int exitFromInt(int i)
{
	loginfo("  Termination requested (signal %d)\n",i);
	return(-1);
}

static int suspendFromInt(int i)
{
	loginfo("  Suspending (signal %d)\n",i);
	raise(SIGSTOP);
	loginfo("  Resuming\n",i);
	return(0);
}

/*
char *logtime()
{
	time_t t;
	struct tm *tm;

	fflush(stdout);
	t = time(NULL);
	tm = (struct tm *)localtime(&t);
	if (tm) {
		if (!strftime(logtimedat,39,"[%c] ",tm)) {
			logtimedat[0] = '\0';
			warning("strftime(..) error");
		}
	}
	else {
		logtimedat[0] = '\0';
		warning("localtime(..) not available");
	}
	fflush(stdout);
	return(logtimedat);
}
*/
