/*
 * This thing parses the commandline args.
 *
 * (c)1999-2000 Frank van Viegen
 * This code is subject to the GPL. Read COPYING for details.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "irmanager.h"
#include "options.h"
#include "text.h"

int optusefifo=0;
int optnodaemon=0;
int optnolocking=0;
int opttestmode=0;
char *optconfig = NULL;
char *optserial = NULL;
char *optlogfile = NULL;
char *optterm = NULL;
int optrepeatdelay = 750;

void parseCmdLine(int argc, char *argv[])
{
	int optno,longopt;
	char *curopt, *s, *tc;
	
	for (optno=1; optno<argc; optno++) {
		curopt = argv[optno];
		if (*curopt!='-') fatalerror("unknown commandline parameter '%s'",argv[optno])
		else {
			for(curopt+=1;*curopt!=(char)0;curopt++) {
				longopt=0;
				switch(*curopt) {
				case 'V' : 
					printinfo(versiontext);
					exit(0);
				case 'f' :
					optusefifo=-1;
					break;
				case 'h' :
					printinfo(helptext);
					exit(0);
				case 'd' :
					optnodaemon=-1;
					break;
				case 'u' :
					optnolocking=-1;
					break;
				case 'l' :
					optno++;
					if (optno>=argc) fatalerror("switch '-l' expects a parameter");
					optlogfile = argv[optno];
					break;
				case 'r' :
					optno++;
					if (optno>=argc) fatalerror("switch '-r' expects a parameter");
					optrepeatdelay = strtol(argv[optno], &tc, 10);
					if (*tc!='\0') fatalerror("'%s' is not a decimal value");
					if (optrepeatdelay<0) warning("using '-r' with a negative value never allows two of the same keys in a row - this could lead to deadlock situations");
					break;
				case 'c' :
					if (optconfig) fatalerror("only one configfile can be specified");
					optno++;
					if (optno>=argc) fatalerror("switch '-c' expects a parameter");
					optconfig = argv[optno];
					break;
				case 'n' :
					optconfig = NULL-1;
					break;
				case 'x' :
					opttestmode = -1;
					optnodaemon = -1;
					if (!optlogfile) optlogfile = "-";
					break;
				case 'T' :
					optno++;
					if (optno>=argc) fatalerror("switch '-T' expects a parameter");
					optterm = argv[optno];
					break;
				case 't' :
					optno++;
					if (optno>=argc) fatalerror("switch '-t' expects a parameter");
					optserial = argv[optno];
					break;
				case '-' :
					longopt=1;
					continue;
				default :
					fatalerror("switch '-%c' unknown",*curopt)
				} /* case*/
				if (longopt) break;
			} /* for */
		} /* if (*s=='-') */
	}

	if (!optconfig) {
		s = getenv("HOME");
		if (!s) s = ".";
		asprintf(&optconfig,"%s/.irmanager/conf",s);
	}
	
	if (!optterm) {
		optterm = getenv("TERM");
		if (!optterm) warning("TERM environment variable not set");
	}
	else {
		if (!strcmp(optterm,"-")) optterm=NULL;
	}
}
