#include <term.h>
#include <string.h>
#include "irmanager.h"
#include "stringtrans.h"

int terminfo=0;

void initStringTrans(char *term)
{
	int e;
	if (term) {
		if (setupterm(term,1,&e)) fatalerror("terminfo can't open terminal information for '%s'",term);
		terminfo=-1;
	}
}

void finishStringTrans()
{
	if (terminfo) {
		del_curterm(cur_term);
		terminfo=0;
	}
}

/* Returns a (freshly allocated) translated string of the original 's'.
 * For speed, it is assumed the 's' string can be written to (although, when
 * the function returns, it will be in its original state.
 */
char *transString(char *s)
{
	static char outstring[1024];
	char *d = outstring;
	char *t, *m;
	int nd = sizeof(outstring)-1;
	int l;
	
	t = strchr(s,'\\');
	while(t) {
		m= NULL;
		l = t-s;
		nd-=l+1;
		if (nd<0) return NULL;
		memcpy(d,s,l);
		d+=l;
		s=t+2;
		switch(*(t+1)) {
		case '\0':
			return NULL;
			break;
		case '\\':
			*d++ = '\\';
			break;
		case 'n':
			*d++ = '\n';
			break;
		case 'r':
			*d++ = '\r';
			break;
		case 't':
			*d++ = '\t';
			break;
		case 'e':
			*d++ = 0x1B; /*escape*/
			break;
		case '[':
			t = strchr(s,']');
			if (!t) return NULL;
			if (terminfo) {
				*t = '\0';
				if (!strcmp(s,"f1")) m = tparm(key_f1);
				else if (!strcmp(s,"f2")) m = tparm(key_f2);
				else if (!strcmp(s,"f3")) m = tparm(key_f3);
				else if (!strcmp(s,"f4")) m = tparm(key_f4);
				else if (!strcmp(s,"f5")) m = tparm(key_f5);
				else if (!strcmp(s,"f6")) m = tparm(key_f6);
				else if (!strcmp(s,"f7")) m = tparm(key_f7);
				else if (!strcmp(s,"f8")) m = tparm(key_f8);
				else if (!strcmp(s,"f9")) m = tparm(key_f9);
				else if (!strcmp(s,"f10")) m = tparm(key_f10);
				else if (!strcmp(s,"f11")) m = tparm(key_f11);
				else if (!strcmp(s,"f12")) m = tparm(key_f12);
				else if (!strcmp(s,"left")) m = tparm(key_left);
				else if (!strcmp(s,"right")) m = tparm(key_right);
				else if (!strcmp(s,"up")) m = tparm(key_up);
				else if (!strcmp(s,"down")) m = tparm(key_down);
				else if (!strcmp(s,"pgdn")) m = tparm(key_npage);
				else if (!strcmp(s,"pgup")) m = tparm(key_ppage);
				else if (!strcmp(s,"home")) m = tparm(key_home);
				else if (!strcmp(s,"end")) m = tparm(key_end);
				else return NULL;
				*t = ']';
			}
			s=t+1;
			break;
		}
		if (m) {
			l=strlen(m);
			nd-=l-1;
			if (nd<0) return NULL;
			memcpy(d,m,l);
			d+=l;
		}
		t = strchr(s,'\\');
	}
	l=strlen(s);
	nd-=l;
	if (nd<0) return NULL;
	memcpy(d,s,l+1);
	return strdup(outstring);
}

/* Test.
#include <stdlib.h>

void main(int c, char *v[])
{
	initStringTrans(getenv("TERM"));
	printf(transString(strdup("test\\[f3]test")));
	finishStringTrans();
}
*/